@@ -10,6 +10,7 @@
 #include <asm/pdc.h>
 #include <asm/byteorder.h>
 #include "load.h"
+#include <stdint.h>
 
 #undef PAGE0
 #define PAGE0   ((struct zeropage *)0x00000000)
@@ -356,6 +357,78 @@ ls(char *path)
      * the current one's data structures ... */
 }
 
+#define ARRAY_SIZE(_x) (sizeof(_x)/sizeof(_x[0]))
+struct nvram_table_map {
+	const char *model;
+	uint64_t tables[2];
+} nvram_table_mapping[] = {
+	{ "9000/785/C8000", { 0xfffffff0f0416848, 0xfffffff0f0416860 } }, // PDC 2.13
+	{ "9000/800/rp34",  { 0xfffffff0f04151f8, 0xfffffff0f0415210 } }, // PDC 46.34 and 50.40
+	{ "9000/800/rp44",  { 0xfffffff0f0420498, 0xfffffff0f04204b0 } }, // PDC 46.34
+};
+
+struct pdc_table {
+	uint16_t id;
+	uint16_t unknown2;
+	uint32_t unknown4;
+	uint64_t nvram;
+	uint64_t defaults;
+};
+
+static uint64_t get_stable_rp34xx(struct nvram_table_map *map, uint16_t id)
+{
+	struct pdc_table *table;
+
+	table = (void *) (uintptr_t) map->tables[0];
+	if (table->id == id) {
+		if (Debug)
+			printf("%04x %016llx %016llx\n", table->id, table->nvram, table->defaults);
+		return (uint64_t)table->nvram;
+	}
+	table = (void *) (uintptr_t) map->tables[1];
+	while (table->id) {
+		if (Debug)
+			printf("%04x %016llx %016llx\n", table->id, table->nvram, table->defaults);
+		if (table->id == id)
+			return (uint64_t)table->nvram;
+		table++;
+	}
+	return 0;
+}
+
+static void enable_mfg_mode(void)
+{
+	const char *model = get_machine_model();
+	struct nvram_table_map *map;
+	unsigned long pdc_vers;
+	int i;
+
+	printf("Machine: %s\n", model);
+	pdc_version(&pdc_vers);
+	printf("Firmware (PDC) version: %d.%d\n", pdc_vers >> 6, pdc_vers & 0x3f);
+
+	for (i = 0; i < ARRAY_SIZE(nvram_table_mapping); i++) {
+		map = nvram_table_mapping + i;
+		if (!strncmp(map->model, model, strlen(map->model)))
+			break;
+	}
+
+	if (i == ARRAY_SIZE(nvram_table_mapping)) {
+		notfound:
+		printf("Don't know how to enable manufacturing mode.\n\n");
+		return;
+	}
+
+	uint64_t addr = get_stable_rp34xx(map, 0x100);
+	if (Debug)
+		printf("got nvram addr %016llx\n", addr);
+	if (!addr)
+		goto notfound;
+	printf("Enabling manufacturing mode and reboot to activate...\n\n", model);
+	*(uint8_t *)(uintptr_t)(addr + 0x20) = 'M';
+	pdc_do_reset();
+}
+
 static void
 interact(int *ok)
 {
@@ -441,16 +514,11 @@ interact(int *ok)
 	if (numbuf[0] == 'x')
 	    pdc_do_reset();
 
-	/* turn on firmware manufacturing mode on C8000 workstation */
+	/* turn on firmware manufacturing mode if possible */
 	if (numbuf[0] == 'm') { /* hidden option! */
-	    const char *model = get_machine_model();
-	    join(commandline, argc, argv, ok);
-	    if (strncmp(model, "9000/785/C8000", 14) == 0) {
-		*(unsigned char *)0xfffffff0f04300a0 = 0x4d;
-		pdc_do_reset();
-	    }
-	    printf("unknown machine - can not enable manufacturing mode\n");
-	    continue;
+		join(commandline, argc, argv, ok);
+		enable_mfg_mode();
+		continue;
 	}
 
 	if (numbuf[0] == 'd') /* hidden option! */
@@ -505,7 +573,6 @@ get_default_serial_console()
 {
 	char *ttyS1_models[] = {
 		"9000/800/rp3410",
-		"9000/800/rp3420",
 		"9000/800/rp3440",
 		NULL
 	};
