@@ -31,19 +31,31 @@ getchar(void)
 static int
 putstring(const char *s)
 {
-    const int len = strlen(s);
+    int len_org = strlen(s);
+    int len;
+    char *p;
 
-    if (len == 0)
-	return 0;
-
-    if (strchr(s, '\n') == NULL)
+    /*
+     * printing single chars is incredibly slow on C8000 machine.
+     * optimize by printing a whole string but take care of new-lines.
+     */
+    do {
+	len = strlen(s);
+	if (len == 0)
+		break;
+	if (*s == '\n') {
+		putchar(*s);
+		s++;
+		continue;
+	}
+	p = strchr(s, '\n');
+	if (p)
+		len = p - s;
         pdc_iodc_cout(s, len);
-    else {
-	while (*s)
-		putchar(*s++);
-    }
+	s += len;
+    } while (len);
 
-    return len;
+    return len_org;
 }
 
 int
