@@ -95,7 +95,7 @@ firmware_call(unsigned long long fn, ...)
     }
     else
     {
-	typedef int (*firmware_entry)();
+	typedef int (*firmware_entry)(unsigned long arg0, ...);
 	unsigned long arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8,
 	    arg9, arg10, arg11, arg12, arg13;
 
@@ -251,6 +251,28 @@ pdc_model_sysmodel(char *name)
     return r;
 }
 
+int
+pdc_version(unsigned long *version)
+{
+    int r;
+
+    r = firmware_call(mem_pdc, PDC_MODEL, PDC_MODEL_VERSIONS, pdc_result, 2);
+    convert_from_wide(pdc_result);
+    if (r == PDC_OK) {
+		if ((pdc_result[0] >> 6) == 0) // old C3000?
+			*version = ((pdc_result[0] >> 4) << 6) | (pdc_result[0] & 0x0f);
+		else
+			*version = pdc_result[0]; /* right-shift by 6 to get major version */
+        } else {
+		/* 32-bit old PDC version */
+		r = firmware_call(mem_pdc, PDC_MODEL, PDC_MODEL_VERSIONS, pdc_result, 1);
+		convert_from_wide(pdc_result);
+		*version = ((pdc_result[0] >> 4) << 6) | (pdc_result[0] & 0x0f);
+        }
+
+    return r;
+}
+
 /*
   See Pdc11-v0.96-Ch3-IODC.pdf at https://parisc.docs.kernel.org/en/latest/technical_documentation.html
   The IODC_FEATURES (byte 10) byte specifies which optional IODC feature are
