@@ -11,7 +11,7 @@
 extern void error(int number, ...);
 
 /* paloio.c */
-extern int seekwrite(int fd, char *buf, unsigned size, unsigned where);
-extern int seekread(int fd, char *buf, unsigned size, unsigned where);
+extern int seekwrite(int fd, char *buf, unsigned size, __u64 where);
+extern int seekread(int fd, char *buf, unsigned size, __u64 where);
 extern int cat(int out, int in);
 extern int fsize(int fd);
